<?php

namespace App\Domain\Elastic\Actions;

use App\Domain\Elastic\Concerns\ElasticIndex;
use App\Domain\Elastic\Models\IndexerTimestamp;
use App\Exceptions\ExceptionFormatter;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Collection;
use Psr\Log\LoggerInterface;
use Throwable;

class DispatchEntityToIndexingAction
{
    public function execute(
        ElasticIndex $index,
        callable $queryBuilder,
        callable $dispatch,
        LoggerInterface $logger
    ): void {
        $logger->info('=== Started scanning for new changes for ' . $index::class);

        try {
            /** @var IndexerTimestamp|null $lastIndexer */
            $lastIndexer = IndexerTimestamp::query()->whereIndex($index)->first();
            $now = now();

            if ($lastIndexer) {
                $logger->info('Looking for records from ' . $lastIndexer->last_schedule->subSeconds(2)->toISOString());
            } else {
                $lastIndexer = IndexerTimestamp::makeForIndex($index);
            }
            $logger->info('Looking for records to ' . $now->toISOString());

            /** @var Builder $query */
            $query = $queryBuilder($lastIndexer->last_schedule, $now);

            $query->chunk(500, function (Collection $results) use ($logger, $dispatch) {
                $ids = $results->pluck('dispatch_id');

                try {
                    $logger->info("Dispatching for indexing for: {$ids->count()} record(s).", [
                        'ids' => $ids->all(),
                    ]);
                    $dispatch($ids->all());
                    $logger->info('Dispatched.');
                } catch (Throwable $e) {
                    $logger->error(ExceptionFormatter::line('Error occurred while dispatching for indexing', $e));
                }
            });

            $lastIndexer->last_schedule = $now;
            $lastIndexer->save();
        } catch (Throwable $e) {
            $logger->error(ExceptionFormatter::line('Error occurred while scanning for changes for indexing', $e));
        }

        $logger->info('=== Finished scanning for new changes');
    }
}
