<?php

use App\Domain\Kafka\Actions\Listen\ListenActualCategoryPropertyAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ActualCategoryProperty\ActualCategoryPropertyEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Offers\Models\ActualCategoryProperty;
use Ensi\LaravelTestFactories\FakerProvider;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-actual-category-property');

test("Action ListenActualCategoryPropertyAction create success", function () {
    /** @var IntegrationTestCase $this */
    $actualCategoryPropertyId = 1;
    $propertyId = 1;
    $categoryId = 1;
    $message = ActualCategoryPropertyEventMessage::factory()
        ->attributes(['id' => $actualCategoryPropertyId, 'property_id' => $propertyId, 'category_id' => $categoryId])
        ->event(ModelEventMessage::CREATE)
        ->make();

    // Create offer that's linked to category and must be marked for indexing
    $offerMark = OfferLinkedFactory::createFromCategoryAndProperty($categoryId, $propertyId);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::createFromCategoryAndProperty($categoryId, $propertyId + 1);

    resolve(ListenActualCategoryPropertyAction::class)->execute($message);

    assertDatabaseHas(ActualCategoryProperty::class, [
        'actual_category_property_id' => $actualCategoryPropertyId,
    ]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');
});

test("Action ListenActualCategoryPropertyAction update success", function (array $dirty, bool $isDirty) {
    /** @var IntegrationTestCase $this */
    /** @var ActualCategoryProperty $actualCategoryProperty */
    $actualCategoryProperty = ActualCategoryProperty::factory()->create();
    $oldUpdatedAt = $actualCategoryProperty->updated_at;
    $newUpdatedAt = $actualCategoryProperty->updated_at->subDay();
    $message = ActualCategoryPropertyEventMessage::factory()
        ->attributes([
            'id' => $actualCategoryProperty->actual_category_property_id,
            'property_id' => $actualCategoryProperty->property_id,
            'category_id' => $actualCategoryProperty->category_id,
            'is_gluing' => !$actualCategoryProperty->is_gluing,
            'updated_at' => $newUpdatedAt->toJSON(),
        ])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that's linked to category and has needed property and must be marked for indexing
    $offerMark = OfferLinkedFactory::createFromCategoryAndProperty($actualCategoryProperty->category_id, $actualCategoryProperty->property_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::createFromCategoryAndProperty($actualCategoryProperty->category_id, $actualCategoryProperty->property_id + 1);

    resolve(ListenActualCategoryPropertyAction::class)->execute($message);

    // updated_at is not dependent on model
    $actualCategoryProperty->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $actualCategoryProperty->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $actualCategoryProperty->updated_at);
    }

    // If there was no update, there must be no mark
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');
})->with([
    [['is_gluing'], true],
    [['is_required'], false],
]);

test("Action ListenActualCategoryPropertyAction delete success", function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var ActualCategoryProperty $actualCategoryProperty */
    $actualCategoryProperty = ActualCategoryProperty::factory()->create();
    $message = ActualCategoryPropertyEventMessage::factory()
        ->attributes([
            'id' => $actualCategoryProperty->actual_category_property_id,
        ])
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create offer that's linked to category and has needed property and must not be marked for indexing
    // because no mark on delete
    $offer = OfferLinkedFactory::createFromCategoryAndProperty($actualCategoryProperty->category_id, $actualCategoryProperty->property_id);

    resolve(ListenActualCategoryPropertyAction::class)->execute($message);

    assertDatabaseMissing(ActualCategoryProperty::class, ['actual_category_property_id' => $actualCategoryProperty->actual_category_property_id]);
    // Offer must not be marked
    assertNewModelFieldEquals($offer, 'updated_at');
})->with(FakerProvider::$optionalDataset);
