<?php

use App\Domain\Kafka\Actions\Listen\ListenBrandAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\Brand\BrandEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Offers\Models\Brand;
use Ensi\LaravelTestFactories\FakerProvider;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-brand');

test("Action ListenBrandAction create success", function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    $brandId = 1;
    $message = BrandEventMessage::factory()->attributes(['id' => $brandId])->event(ModelEventMessage::CREATE)->make();

    // Create offer that's linked to category and must be marked for indexing
    $offerMark = OfferLinkedFactory::createFromBrand($brandId);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::createFromBrand($brandId + 1);

    resolve(ListenBrandAction::class)->execute($message);

    assertDatabaseHas(Brand::class, [
        'brand_id' => $brandId,
    ]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');
})->with(FakerProvider::$optionalDataset);

test("Action ListenBrandAction update success", function (array $dirty, bool $isDirty, ?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var Brand $brand */
    $brand = Brand::factory()->create();
    $oldUpdatedAt = $brand->updated_at;
    $newUpdatedAt = $brand->updated_at->subDay();
    $message = BrandEventMessage::factory()
        ->attributes(['id' => $brand->brand_id, 'name' => $brand->name . '_1', 'updated_at' => $newUpdatedAt->toJSON()])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that's linked to category and has needed brand and must be marked for indexing
    $offerMark = OfferLinkedFactory::createFromBrand($brand->brand_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::createFromBrand($brand->brand_id + 1);

    resolve(ListenBrandAction::class)->execute($message);

    // updated_at is not dependent on model
    $brand->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $brand->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $brand->updated_at);
    }

    // If there was no update, there must be no mark
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');
})->with([
    [['name'], true],
    [['is_active'], false],
], FakerProvider::$optionalDataset);

test("Action ListenBrandAction delete success", function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var Brand $brand */
    $brand = Brand::factory()->create();
    $message = BrandEventMessage::factory()
        ->attributes(['id' => $brand->brand_id])
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create offer that's linked to category and has needed brand and must not be marked for indexing
    // because no mark on delete
    $offer = OfferLinkedFactory::createFromBrand($brand->brand_id);

    resolve(ListenBrandAction::class)->execute($message);

    assertDatabaseMissing(Brand::class, ['brand_id' => $brand->brand_id]);
    // Offer must not be marked
    assertNewModelFieldEquals($offer, 'updated_at');
})->with(FakerProvider::$optionalDataset);
