<?php

use App\Domain\Kafka\Actions\Listen\ListenCategoryAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\Category\CategoryEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Offers\Models\Category;
use Ensi\LaravelTestFactories\FakerProvider;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-category');

test("Action ListenCategoryAction create success", function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    $categoryId = 1;
    $message = CategoryEventMessage::factory()->attributes(['id' => $categoryId])->event(ModelEventMessage::CREATE)->make();

    // Create offer that's linked to category and must be marked for indexing
    $offerMark = OfferLinkedFactory::createFromCategory($categoryId);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::createFromCategory($categoryId + 1);

    resolve(ListenCategoryAction::class)->execute($message);

    assertDatabaseHas(Category::class, [
        'category_id' => $categoryId,
    ]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');
})->with(FakerProvider::$optionalDataset);

test("Action ListenCategoryAction update success", function (array $dirty, bool $isDirty, ?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var Category $category */
    $category = Category::factory()->create();
    $oldUpdatedAt = $category->updated_at;
    $newUpdatedAt = $category->updated_at->subDay();
    $message = CategoryEventMessage::factory()
        ->attributes(['id' => $category->category_id, 'updated_at' => $newUpdatedAt->toJSON()])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that's linked to category and must be marked for indexing
    $offerMark = OfferLinkedFactory::createFromCategory($category->category_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::createFromCategory($category->category_id + 1);

    resolve(ListenCategoryAction::class)->execute($message);

    // updated_at is not dependent on model
    $category->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $category->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $category->updated_at);
    }

    // If there was no update, there must be no mark
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');
})->with([
    [['name'], true],
    [['is_active'], false],
], FakerProvider::$optionalDataset);

test("Action ListenCategoryAction delete success", function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var Category $category */
    $category = Category::factory()->create();
    $message = CategoryEventMessage::factory()
        ->attributes(['id' => $category->category_id])
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create offer that's linked to category and must not be marked for indexing
    // because no mark on delete
    $offer = OfferLinkedFactory::createFromCategory($category->category_id);

    resolve(ListenCategoryAction::class)->execute($message);

    assertDatabaseMissing(Category::class, ['category_id' => $category->category_id]);
    // Offer must not be marked
    assertNewModelFieldEquals($offer, 'updated_at');
})->with(FakerProvider::$optionalDataset);
