<?php

namespace App\Domain\Kafka\Actions\Tests;

use App\Domain\Kafka\Actions\Listen\ListenCategoryProductLinkAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\CategoryProductLink\CategoryProductLinkEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Offers\Models\CategoryProductLink;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka');

test("Action ListenCategoryProductLinkAction create success", function () {
    /** @var IntegrationTestCase $this */
    $categoryProductId = 1;
    $productId = 1;
    $message = CategoryProductLinkEventMessage::factory()
        ->attributes(['id' => $categoryProductId, 'product_id' => $productId])
        ->event(ModelEventMessage::CREATE)
        ->make();

    // Create offer that's linked to category and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($productId);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::createFromCategory($productId + 1);

    resolve(ListenCategoryProductLinkAction::class)->execute($message);

    assertDatabaseHas(CategoryProductLink::class, [
        'category_product_id' => $categoryProductId,
    ]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');
});

test("Action ListenCategoryProductLinkAction update success", function (array $dirty, bool $isDirty) {
    /** @var IntegrationTestCase $this */
    /** @var CategoryProductLink $link */
    $link = CategoryProductLink::factory()->create();
    $oldUpdatedAt = $link->updated_at;
    $newUpdatedAt = $link->updated_at->subDay();
    $message = CategoryProductLinkEventMessage::factory()
        ->attributes([
            'id' => $link->category_product_id,
            'product_id' => $link->product_id,
            'updated_at' => $newUpdatedAt->toJSON(),
        ])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that's linked to category and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($link->product_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($link->product_id + 1);

    resolve(ListenCategoryProductLinkAction::class)->execute($message);

    // updated_at is not dependent on model
    $link->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $link->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $link->updated_at);
    }

    // If save wasn't triggered, product must not be marked for indexing
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');
})->with([
    [['product_id'], true],
    [['category_id'], true],
]);

test("Action ListenCategoryProductLinkAction delete success", function () {
    /** @var IntegrationTestCase $this */
    /** @var CategoryProductLink $link */
    $link = CategoryProductLink::factory()->create();
    $message = CategoryProductLinkEventMessage::factory()
        ->attributes([
            'id' => $link->category_product_id,
            'product_id' => $link->product_id,
        ])
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create offer that's linked to category and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($link->product_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($link->product_id + 1);

    resolve(ListenCategoryProductLinkAction::class)->execute($message);

    assertDatabaseMissing(CategoryProductLink::class, ['category_product_id' => $link->category_product_id]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');
});
