<?php

use App\Domain\Kafka\Actions\Listen\ListenNameplateAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\Nameplate\NameplateEventMessage;
use App\Domain\Offers\Models\Nameplate;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka');

test("Action ListenNameplateAction create success", function () {
    /** @var IntegrationTestCase $this */
    $nameplateId = 1;
    $message = NameplateEventMessage::factory()
        ->attributes([
            'id' => $nameplateId,
        ])
        ->event(ModelEventMessage::CREATE)
        ->make();

    resolve(ListenNameplateAction::class)->execute($message);

    assertDatabaseHas(Nameplate::class, ['nameplate_id' => $nameplateId]);
});

test("Action ListenNameplateAction update success", function (array $dirty, bool $isDirty) {
    /** @var IntegrationTestCase $this */
    /** @var Nameplate $nameplate */
    $nameplate = Nameplate::factory()->create();
    $oldUpdatedAt = $nameplate->updated_at;
    $newUpdatedAt = $nameplate->updated_at->subDay();
    $message = NameplateEventMessage::factory()
        ->attributes([
            'id' => $nameplate->nameplate_id,
            'is_active' => true,
            'updated_at' => $newUpdatedAt->toJSON(),
        ])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that's linked to category and has needed nameplate and must be marked for indexing
    $offerMark = OfferLinkedFactory::createFromNameplate($nameplate->nameplate_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::createFromNameplate($nameplate->nameplate_id + 1);

    resolve(ListenNameplateAction::class)->execute($message);

    // updated_at is not dependent on model
    $nameplate->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $nameplate->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $nameplate->updated_at);
    }

    // If there was no update, there must be no mark
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');
})->with([
    [['name'], true],
    [['code'], true],
    [['background_color'], true],
    [['text_color'], true],
    [['is_active'], true],
]);

test("Action ListenNameplateAction delete success", function () {
    /** @var IntegrationTestCase $this */
    /** @var Nameplate $nameplate */
    $nameplate = Nameplate::factory()->create();
    $message = NameplateEventMessage::factory()
        ->attributes(['id' => $nameplate->nameplate_id])
        ->event(ModelEventMessage::DELETE)
        ->make();

    resolve(ListenNameplateAction::class)->execute($message);

    assertDatabaseMissing(Nameplate::class, ['nameplate_id' => $nameplate->nameplate_id]);
});
