<?php

namespace App\Domain\Kafka\Actions\Tests;

use App\Domain\Kafka\Actions\Listen\ListenNameplateProductAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\NameplateProduct\NameplateProductEventMessage;
use App\Domain\Offers\Models\NameplateProduct;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka');

test("Action ListenNameplateProductAction create success", function () {
    /** @var IntegrationTestCase $this */
    $nameplateProductId = 1;
    $productId = 1;
    $message = NameplateProductEventMessage::factory()
        ->attributes(['id' => $nameplateProductId, 'product_id' => $productId])
        ->event(ModelEventMessage::CREATE)
        ->make();

    // Create offer that must be marked for indexing
    $offerMark = OfferLinkedFactory::create($productId);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($productId + 1);

    resolve(ListenNameplateProductAction::class)->execute($message);

    assertDatabaseHas(NameplateProduct::class, [
        'nameplate_product_id' => $nameplateProductId,
    ]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');
});

test("Action ListenNameplateProductAction update success", function (array $dirty, bool $isDirty) {
    /** @var IntegrationTestCase $this */
    /** @var NameplateProduct $nameplateProduct */
    $nameplateProduct = NameplateProduct::factory()->create();
    $oldUpdatedAt = $nameplateProduct->updated_at;
    $newUpdatedAt = $nameplateProduct->updated_at->subDay();
    $message = NameplateProductEventMessage::factory()
        ->attributes([
            'id' => $nameplateProduct->nameplate_product_id,
            'product_id' => $nameplateProduct->product_id,
            'updated_at' => $newUpdatedAt->toJSON(),
        ])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that has needed nameplate and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($nameplateProduct->product_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($nameplateProduct->product_id + 1);

    resolve(ListenNameplateProductAction::class)->execute($message);

    // updated_at is not dependent on model
    $nameplateProduct->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $nameplateProduct->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $nameplateProduct->updated_at);
    }

    // If there was no update, there must be no mark
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');
})->with([
    [['product_id'], true],
    [['nameplate_id'], true],
]);

test("Action ListenNameplateProductAction delete success", function () {
    /** @var IntegrationTestCase $this */
    /** @var NameplateProduct $nameplateProduct */
    $nameplateProduct = NameplateProduct::factory()->create();
    $message = NameplateProductEventMessage::factory()
        ->attributes([
            'id' => $nameplateProduct->nameplate_product_id,
            'product_id' => $nameplateProduct->product_id,
        ])
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create offer that has needed nameplate and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($nameplateProduct->product_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($nameplateProduct->product_id + 1);

    resolve(ListenNameplateProductAction::class)->execute($message);

    assertDatabaseMissing(NameplateProduct::class, ['nameplate_product_id' => $nameplateProduct->nameplate_product_id]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');
});
