<?php

use App\Domain\Kafka\Actions\Listen\ListenProductGroupAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ProductGroup\ProductGroupEventMessage;
use App\Domain\Offers\Models\ProductGroup;
use Ensi\LaravelTestFactories\FakerProvider;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-product-group');

test("Action ListenProductGroupAction create success", function () {
    /** @var IntegrationTestCase $this */
    $productGroupId = 1;
    $message = ProductGroupEventMessage::factory()
        ->attributes(['id' => $productGroupId])
        ->event(ModelEventMessage::CREATE)
        ->make();

    // Create offer that's linked to product group and must be marked for indexing
    $offerMark = OfferLinkedFactory::create();
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($offerMark->product_id + 1);

    // Prepare offers in one product group + link $offerMark with it
    $groupOffer = OfferLinkedFactory::createGroupedProduct($offerMark->product_id, $productGroupId);
    $otherOffer1 = OfferLinkedFactory::create();
    $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);

    resolve(ListenProductGroupAction::class)->execute($message);

    assertDatabaseHas(ProductGroup::class, [
        'product_group_id' => $productGroupId,
    ]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');

    // Validate that offers were marked correctly
    assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
    assertNewModelFieldEquals($otherOffer1, 'updated_at');
    assertNewModelFieldEquals($otherOffer2, 'updated_at');
});

test("Action ListenProductGroupAction update success", function (array $dirty, bool $isDirty) {
    /** @var IntegrationTestCase $this */
    /** @var ProductGroup $productGroup */
    $productGroup = ProductGroup::factory()->create();
    $oldUpdatedAt = $productGroup->updated_at;
    $newUpdatedAt = $productGroup->updated_at->subDay();
    $message = ProductGroupEventMessage::factory()
        ->attributes([
            'id' => $productGroup->product_group_id,
            'updated_at' => $newUpdatedAt->toJSON(),
        ])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that's linked to product group and must be marked for indexing
    $offerMark = OfferLinkedFactory::create();
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($offerMark->product_id + 1);

    // Prepare offers in one product group + link $offerMark with it
    $groupOffer = OfferLinkedFactory::createGroupedProduct($offerMark->product_id, $productGroup->product_group_id);
    $otherOffer1 = OfferLinkedFactory::create();
    $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);

    resolve(ListenProductGroupAction::class)->execute($message);

    // updated_at is not dependent on model
    $productGroup->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $productGroup->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $productGroup->updated_at);
    }

    // If there was no update, there must be no mark
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');

    // Validate that offers were marked correctly
    if ($isDirty) {
        assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
    } else {
        assertNewModelFieldEquals($groupOffer, 'updated_at');
    }
    assertNewModelFieldEquals($otherOffer1, 'updated_at');
    assertNewModelFieldEquals($otherOffer2, 'updated_at');
})->with([
    [['name'], true],
    [['is_active'], true],
    [['product_id'], false],
]);

test("Action ListenProductGroupAction delete success", function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var ProductGroup $productGroup */
    $productGroup = ProductGroup::factory()->create();
    $message = ProductGroupEventMessage::factory()
        ->attributes([
            'id' => $productGroup->product_group_id,
        ])
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create offer that's linked to product group and must be marked for indexing
    $offerMark = OfferLinkedFactory::create();
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($offerMark->product_id + 1);

    // Prepare offers in one product group + link $offerMark with it
    $groupOffer = OfferLinkedFactory::createGroupedProduct($offerMark->product_id, $productGroup->product_group_id);
    $otherOffer1 = OfferLinkedFactory::create();
    $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);

    resolve(ListenProductGroupAction::class)->execute($message);

    assertDatabaseMissing(ProductGroup::class, ['product_group_id' => $productGroup->product_group_id]);
    // Offers must not be marked, as we await ProductGroupProduct event
    assertNewModelFieldEquals($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');

    // Validate that offers in one product group were NOT marked
    assertNewModelFieldEquals($groupOffer, 'updated_at');
    assertNewModelFieldEquals($otherOffer1, 'updated_at');
    assertNewModelFieldEquals($otherOffer2, 'updated_at');
})->with(FakerProvider::$optionalDataset);
