<?php

use App\Domain\Kafka\Actions\Listen\ListenProductGroupProductAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ProductGroupProduct\ProductGroupProductEventMessage;
use App\Domain\Offers\Models\ProductGroupProduct;
use Ensi\LaravelTestFactories\FakerProvider;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-product-group-product');

test("Action ListenProductGroupProductAction create success", function () {
    /** @var IntegrationTestCase $this */

    $productGroupProductId = 1;
    $productGroupId = 2;
    $productId = 3;
    $message = ProductGroupProductEventMessage::factory()
        ->attributes([
            'id' => $productGroupProductId,
            'product_id' => $productId,
            'product_group_id' => $productGroupId,
        ])
        ->event(ModelEventMessage::CREATE)
        ->make();

    // Create offer that's linked to product group and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($productId);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($productId + 1);

    // Prepare offer in one product group with new one
    $groupOffer = OfferLinkedFactory::createGroupedProduct(productGroupId: $productGroupId);
    $otherOffer1 = OfferLinkedFactory::create();
    $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);

    resolve(ListenProductGroupProductAction::class)->execute($message);

    assertDatabaseHas(ProductGroupProduct::class, [
        'product_group_product_id' => $productGroupProductId,
    ]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');

    // Validate that offers were marked correctly
    assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
    assertNewModelFieldEquals($otherOffer1, 'updated_at');
    assertNewModelFieldEquals($otherOffer2, 'updated_at');
});

test("Action ListenProductGroupProductAction update success", function (array $dirty, bool $isDirty) {
    /** @var IntegrationTestCase $this */

    /** @var ProductGroupProduct $productGroupProduct */
    $productGroupProduct = ProductGroupProduct::factory()->create();
    $oldUpdatedAt = $productGroupProduct->updated_at;
    $newUpdatedAt = $productGroupProduct->updated_at->subDay();
    $message = ProductGroupProductEventMessage::factory()
        ->attributes([
            'id' => $productGroupProduct->product_group_product_id,
            'product_id' => $productGroupProduct->product_id + 2,
            'product_group_id' => $productGroupProduct->product_group_id,
            'updated_at' => $newUpdatedAt->toJSON(),
        ])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that's linked to product group and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($productGroupProduct->product_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($productGroupProduct->product_id + 1);

    // Prepare offer in one product group with new one
    $groupOffer = OfferLinkedFactory::createGroupedProduct(productGroupId: $productGroupProduct->product_group_id);
    $otherOffer1 = OfferLinkedFactory::create();
    $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);

    resolve(ListenProductGroupProductAction::class)->execute($message);

    // updated_at is not dependent on model
    $productGroupProduct->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $productGroupProduct->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $productGroupProduct->updated_at);
    }

    // If there was no update, there must be no mark
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');

    // Validate that offers were marked correctly
    if ($isDirty) {
        assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
    } else {
        assertNewModelFieldEquals($groupOffer, 'updated_at');
    }
    assertNewModelFieldEquals($otherOffer1, 'updated_at');
    assertNewModelFieldEquals($otherOffer2, 'updated_at');
})->with([
    [['product_id'], true],
    [['is_active'], false],
]);

test("Action ListenProductGroupProductAction delete success", function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var ProductGroupProduct $productGroupProduct */
    $productGroupProduct = ProductGroupProduct::factory()->create();
    $message = ProductGroupProductEventMessage::factory()
        ->attributes([
            'id' => $productGroupProduct->product_group_product_id,
        ])
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create offer that's linked to product group and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($productGroupProduct->product_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($productGroupProduct->product_id + 1);

    // Prepare offer in one product group with new one
    $groupOffer = OfferLinkedFactory::createGroupedProduct(productGroupId: $productGroupProduct->product_group_id);
    $otherOffer1 = OfferLinkedFactory::create();
    $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);

    resolve(ListenProductGroupProductAction::class)->execute($message);

    assertDatabaseMissing(ProductGroupProduct::class, ['product_group_product_id' => $productGroupProduct->product_group_product_id]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');

    // Validate that offers were marked correctly
    assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
    assertNewModelFieldEquals($otherOffer1, 'updated_at');
    assertNewModelFieldEquals($otherOffer2, 'updated_at');
})->with(FakerProvider::$optionalDataset)->group('t');
