<?php

use App\Domain\Kafka\Actions\Listen\ListenPropertyDirectoryValueAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\PropertyDirectoryValue\PropertyDirectoryValueEventMessage;
use App\Domain\Offers\Models\PropertyDirectoryValue;
use Ensi\LaravelTestFactories\FakerProvider;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-property-directory-value');

test("Action ListenPropertyDirectoryValueAction create success", function (?bool $always, bool $group) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    $propertyDirectoryValueId = 1;
    $message = PropertyDirectoryValueEventMessage::factory()->attributes(['id' => $propertyDirectoryValueId])->event(ModelEventMessage::CREATE)->make();

    // Create offer that's linked to PropertyDirectoryValue and must be marked for indexing
    $offerMark = OfferLinkedFactory::createFromDirectoryValue($propertyDirectoryValueId);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::createFromDirectoryValue($propertyDirectoryValueId + 1);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($offerMark->product_id);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPropertyDirectoryValueAction::class)->execute($message);

    assertDatabaseHas(PropertyDirectoryValue::class, [
        'directory_value_id' => $propertyDirectoryValueId,
    ]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');

    if ($group) {
        // Validate that offers were marked correctly
        assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
        assertNewModelFieldEquals($otherOffer1, 'updated_at');
        assertNewModelFieldEquals($otherOffer2, 'updated_at');
    }
})->with(FakerProvider::$optionalDataset)->with([true, false]);

test("Action ListenPropertyDirectoryValueAction update success", function (
    array $dirty,
    bool $isDirty,
    ?bool $always,
    bool $group,
) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var PropertyDirectoryValue $propertyDirectoryValue */
    $propertyDirectoryValue = PropertyDirectoryValue::factory()->create();
    $oldUpdatedAt = $propertyDirectoryValue->updated_at;
    $newUpdatedAt = $propertyDirectoryValue->updated_at->subDay();
    $message = PropertyDirectoryValueEventMessage::factory()
        ->attributes(['id' => $propertyDirectoryValue->directory_value_id, 'updated_at' => $newUpdatedAt->toJSON()])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that's linked to PropertyDirectoryValue and must be marked for indexing
    $offerMark = OfferLinkedFactory::createFromDirectoryValue($propertyDirectoryValue->directory_value_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::createFromDirectoryValue($propertyDirectoryValue->directory_value_id + 1);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($offerMark->product_id);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPropertyDirectoryValueAction::class)->execute($message);

    // updated_at is not dependent on model
    $propertyDirectoryValue->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $propertyDirectoryValue->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $propertyDirectoryValue->updated_at);
    }

    // If there was no update, there must be no mark
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');

    if ($group) {
        // Validate that offers were marked correctly
        if ($isDirty) {
            assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
        } else {
            assertNewModelFieldEquals($groupOffer, 'updated_at');
        }
        assertNewModelFieldEquals($otherOffer1, 'updated_at');
        assertNewModelFieldEquals($otherOffer2, 'updated_at');
    }
})->with([
    [['name'], true],
    [['undefined'], false],
], FakerProvider::$optionalDataset, [true, false]);

test("Action ListenPropertyDirectoryValueAction delete success", function (?bool $always, bool $group) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var PropertyDirectoryValue $propertyDirectoryValue */
    $propertyDirectoryValue = PropertyDirectoryValue::factory()->create();
    $message = PropertyDirectoryValueEventMessage::factory()
        ->attributes(['id' => $propertyDirectoryValue->directory_value_id])
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create offer that's linked to PropertyDirectoryValue and must not be marked for indexing
    // because no mark on delete
    $offer = OfferLinkedFactory::createFromDirectoryValue($propertyDirectoryValue->directory_value_id);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($offer->product_id);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPropertyDirectoryValueAction::class)->execute($message);

    assertDatabaseMissing(PropertyDirectoryValue::class, ['directory_value_id' => $propertyDirectoryValue->directory_value_id]);

    // Offer must not be marked
    assertNewModelFieldEquals($offer, 'updated_at');

    if ($group) {
        // Validate that offers were NOT marked
        assertNewModelFieldEquals($groupOffer, 'updated_at');
        assertNewModelFieldEquals($otherOffer1, 'updated_at');
        assertNewModelFieldEquals($otherOffer2, 'updated_at');
    }
})->with(FakerProvider::$optionalDataset)->with([true, false]);
