<?php

use App\Domain\Kafka\Actions\Listen\ListenPropertyAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\Property\PropertyEventMessage;
use App\Domain\Offers\Models\Property;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-property');

test("Action ListenPropertyAction create success", function (bool $group) {
    /** @var IntegrationTestCase $this */
    $propertyId = 1;
    $message = PropertyEventMessage::factory()->attributes(['id' => $propertyId])->event(ModelEventMessage::CREATE)->make();

    // Create offer that's linked to property and must be marked for indexing
    $offerMark = OfferLinkedFactory::createFromProperty($propertyId);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::createFromProperty($propertyId + 1);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($offerMark->product_id);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPropertyAction::class)->execute($message);

    assertDatabaseHas(Property::class, [
        'property_id' => $propertyId,
    ]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');

    if ($group) {
        // Validate that offers were marked correctly
        assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
        assertNewModelFieldEquals($otherOffer1, 'updated_at');
        assertNewModelFieldEquals($otherOffer2, 'updated_at');
    }
})->with([true, false]);

test("Action ListenPropertyAction update success", function (
    array $dirty,
    bool $isDirty,
    bool $group,
) {
    /** @var IntegrationTestCase $this */
    /** @var Property $property */
    $property = Property::factory()->create();
    $oldUpdatedAt = $property->updated_at;
    $newUpdatedAt = $property->updated_at->subDay();
    $message = PropertyEventMessage::factory()
        ->attributes(['id' => $property->property_id, 'updated_at' => $newUpdatedAt->toJSON()])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that's linked to property and must be marked for indexing
    $offerMark = OfferLinkedFactory::createFromProperty($property->property_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::createFromProperty($property->property_id + 1);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($offerMark->product_id);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPropertyAction::class)->execute($message);

    // updated_at is not dependent on model
    $property->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $property->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $property->updated_at);
    }

    // If there was no update, there must be no mark
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');

    if ($group) {
        // Validate that offers were marked correctly
        if ($isDirty) {
            assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
        } else {
            assertNewModelFieldEquals($groupOffer, 'updated_at');
        }
        assertNewModelFieldEquals($otherOffer1, 'updated_at');
        assertNewModelFieldEquals($otherOffer2, 'updated_at');
    }
})->with([
    [['display_name'], true],
    [['name'], false],
], [true, false]);

test("Action ListenPropertyAction delete success", function (bool $group) {
    /** @var IntegrationTestCase $this */
    /** @var Property $property */
    $property = Property::factory()->create();
    $message = PropertyEventMessage::factory()
        ->attributes(['id' => $property->property_id])
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create offer that's linked to property and must not be marked for indexing
    // because no mark on delete
    $offer = OfferLinkedFactory::createFromProperty($property->property_id);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($offer->product_id);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPropertyAction::class)->execute($message);

    assertDatabaseMissing(Property::class, ['property_id' => $property->property_id]);

    // Offer must not be marked
    assertNewModelFieldEquals($offer, 'updated_at');

    if ($group) {
        // Validate that offers were NOT marked
        assertNewModelFieldEquals($groupOffer, 'updated_at');
        assertNewModelFieldEquals($otherOffer1, 'updated_at');
        assertNewModelFieldEquals($otherOffer2, 'updated_at');
    }
})->with([true, false]);
