<?php

use App\Domain\Kafka\Actions\Listen\ListenPublishedImageAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\PublishedImage\PublishedImageEventMessage;
use App\Domain\Offers\Models\Image;
use Ensi\LaravelTestFactories\FakerProvider;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-published-image');

test("Action ListenPublishedImageAction create success", function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    $imageId = 1;
    $productId = 1;
    $message = PublishedImageEventMessage::factory()
        ->attributes(['id' => $imageId, 'product_id' => $productId])
        ->event(ModelEventMessage::CREATE)
        ->make();

    // Create offer that's linked to image and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($productId);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($productId + 1);

    resolve(ListenPublishedImageAction::class)->execute($message);

    assertDatabaseHas(Image::class, [
        'image_id' => $imageId,
    ]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');
})->with(FakerProvider::$optionalDataset);

test("Action ListenPublishedImageAction update success", function (array $dirty, bool $isDirty, ?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var Image $image */
    $image = Image::factory()->create();
    $oldUpdatedAt = $image->updated_at;
    $newUpdatedAt = $image->updated_at->subDay();
    $message = PublishedImageEventMessage::factory()
        ->attributes(['id' => $image->image_id, 'updated_at' => $newUpdatedAt->toJSON(), 'product_id' => $image->product_id])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that's linked to image and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($image->product_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($image->product_id + 1);

    resolve(ListenPublishedImageAction::class)->execute($message);

    // updated_at is not dependent on model
    $image->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $image->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $image->updated_at);
    }

    // Validate that offers were marked correctly
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');
})->with([
    [['name'], true],
    [['is_active'], false],
], FakerProvider::$optionalDataset);

test("Action ListenPublishedImageAction delete success", function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var Image $image */
    $image = Image::factory()->create();
    $message = PublishedImageEventMessage::factory()
        ->attributes(['id' => $image->image_id])
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create offer that's linked to image and must be marked for indexing
    $offer = OfferLinkedFactory::create($image->product_id);

    resolve(ListenPublishedImageAction::class)->execute($message);

    assertDatabaseMissing(Image::class, ['image_id' => $image->image_id]);
    assertNewModelFieldGreaterThan($offer, 'updated_at');
})->with(FakerProvider::$optionalDataset);
