<?php

use App\Domain\Kafka\Actions\Listen\ListenPublishedProductAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\PublishedProduct\PublishedProductEventMessage;
use App\Domain\Offers\Models\Product;
use Ensi\LaravelTestFactories\FakerProvider;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-published-product');

test("Action ListenPublishedProductAction create success", function (bool $allowPublish, ?bool $always, bool $group) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    $productId = 1;
    $message = PublishedProductEventMessage::factory()->attributes(['id' => $productId, 'allow_publish' => $allowPublish])->event(ModelEventMessage::CREATE)->make();

    // Create offer that's linked to product and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($productId);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($productId + 1);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($productId);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPublishedProductAction::class)->execute($message);

    if ($allowPublish) {
        assertDatabaseHas(Product::class, ['product_id' => $productId]);
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
        if ($group) {
            // Validate that offers were marked correctly
            assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
            assertNewModelFieldEquals($otherOffer1, 'updated_at');
            assertNewModelFieldEquals($otherOffer2, 'updated_at');
        }
    } else {
        assertDatabaseMissing(Product::class, ['product_id' => $productId]);
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
})->with([true, false], FakerProvider::$optionalDataset, [true, false]);

test("Action ListenPublishedProductAction update success", function (
    array $dirty,
    bool $isDirty,
    bool $allowPublish,
    ?bool $always,
    bool $group,
) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var Product $product */
    $product = Product::factory()->create();
    $oldUpdatedAt = $product->updated_at;
    $newUpdatedAt = $product->updated_at->subDay();
    $message = PublishedProductEventMessage::factory()
        ->attributes(['id' => $product->product_id, 'allow_publish' => $allowPublish, 'updated_at' => $newUpdatedAt->toJSON()])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that's linked to product and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($product->product_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($product->product_id + 1);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($offerMark->product_id);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPublishedProductAction::class)->execute($message);

    // updated_at is not dependent on model
    $product->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $product->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $product->updated_at);
    }

    // If there was no update, there must be no mark
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');

    if ($group) {
        // Validate that offers were marked correctly
        if ($isDirty) {
            assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
        } else {
            assertNewModelFieldEquals($groupOffer, 'updated_at');
        }
        assertNewModelFieldEquals($otherOffer1, 'updated_at');
        assertNewModelFieldEquals($otherOffer2, 'updated_at');
    }
})->with([
    [['name'], true, true],
    [['is_active'], false, true],
    [['allow_publish'], true, false],
    [[], false, false],
], FakerProvider::$optionalDataset, [true, false]);

test("Action ListenPublishedProductAction delete success", function (?bool $always, bool $group) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var Product $product */
    $product = Product::factory()->create();
    $message = PublishedProductEventMessage::factory()
        ->attributes(['id' => $product->product_id])
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create offer that's linked to product and must be marked for indexing
    $offer = OfferLinkedFactory::create($product->product_id);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($product->product_id);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPublishedProductAction::class)->execute($message);

    assertDatabaseMissing(Product::class, ['product_id' => $product->product_id]);
    assertNewModelFieldGreaterThan($offer, 'updated_at');

    if ($group) {
        // Validate that offers were marked correctly
        assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
        assertNewModelFieldEquals($otherOffer1, 'updated_at');
        assertNewModelFieldEquals($otherOffer2, 'updated_at');
    }
})->with(FakerProvider::$optionalDataset)->with([true, false]);
