<?php

use App\Domain\Kafka\Actions\Listen\ListenPublishedPropertyValueAction;
use App\Domain\Kafka\Actions\Tests\Factories\OfferLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\PublishedPropertyValue\PublishedPropertyValueEventMessage;
use App\Domain\Offers\Models\ProductPropertyValue;
use Ensi\LaravelTestFactories\FakerProvider;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-published-property-value');

test("Action ListenPublishedPropertyValueAction create success", function (?bool $always, bool $group) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    $propertyValueId = 1;
    $productId = 1;
    $message = PublishedPropertyValueEventMessage::factory()
        ->attributes(['id' => $propertyValueId, 'deleted_at' => null, 'product_id' => $productId])
        ->event(ModelEventMessage::CREATE)
        ->make();

    // Create offer that's linked to property value and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($productId);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($productId + 1);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($offerMark->product_id);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPublishedPropertyValueAction::class)->execute($message);

    assertDatabaseHas(ProductPropertyValue::class, [
        'product_property_value_id' => $propertyValueId,
    ]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');

    if ($group) {
        // Validate that offers were marked correctly
        assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
        assertNewModelFieldEquals($otherOffer1, 'updated_at');
        assertNewModelFieldEquals($otherOffer2, 'updated_at');
    }
})->with(FakerProvider::$optionalDataset)->with([true, false]);

test("Action ListenPublishedPropertyValueAction update success", function (
    array $dirty,
    bool $isDirty,
    ?bool $always,
    bool $group,
) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var ProductPropertyValue $propertyValue */
    $propertyValue = ProductPropertyValue::factory()->create();
    $oldUpdatedAt = $propertyValue->updated_at;
    $newUpdatedAt = $propertyValue->updated_at->subDay();
    $message = PublishedPropertyValueEventMessage::factory()
        ->attributes([
            'id' => $propertyValue->product_property_value_id,
            'product_id' => $propertyValue->product_id,
            'updated_at' => $newUpdatedAt->toJSON(),
            'deleted_at' => null,
        ])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    // Create offer that's linked to property value and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($propertyValue->product_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($propertyValue->product_id + 1);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($offerMark->product_id);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPublishedPropertyValueAction::class)->execute($message);

    // updated_at is not dependent on model
    $propertyValue->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $propertyValue->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $propertyValue->updated_at);
    }

    // If there was no update, there must be no mark
    if ($isDirty) {
        assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    } else {
        assertNewModelFieldEquals($offerMark, 'updated_at');
    }
    assertNewModelFieldEquals($offerOther, 'updated_at');

    if ($group) {
        // Validate that offers were marked correctly
        if ($isDirty) {
            assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
        } else {
            assertNewModelFieldEquals($groupOffer, 'updated_at');
        }
        assertNewModelFieldEquals($otherOffer1, 'updated_at');
        assertNewModelFieldEquals($otherOffer2, 'updated_at');
    }
})->with([
    [['name'], true],
    [['undefined'], false],
], FakerProvider::$optionalDataset, [true, false]);

test("Action ListenPublishedPropertyValueAction delete success", function (?bool $always, bool $group) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var ProductPropertyValue $propertyValue */
    $propertyValue = ProductPropertyValue::factory()->create();
    $message = PublishedPropertyValueEventMessage::factory()
        ->attributes(['id' => $propertyValue->product_property_value_id])
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create offer that's linked to property value and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($propertyValue->product_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($propertyValue->product_id + 1);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($offerMark->product_id);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPublishedPropertyValueAction::class)->execute($message);

    assertDatabaseMissing(ProductPropertyValue::class, ['product_property_value_id' => $propertyValue->id]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');

    if ($group) {
        // Validate that offers were marked correctly
        assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
        assertNewModelFieldEquals($otherOffer1, 'updated_at');
        assertNewModelFieldEquals($otherOffer2, 'updated_at');
    }
})->with(FakerProvider::$optionalDataset)->with([true, false]);

test("Action ListenPublishedPropertyValueAction delete_at success", function (?bool $always, bool $group) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var ProductPropertyValue $propertyValue */
    $propertyValue = ProductPropertyValue::factory()->create();
    $message = PublishedPropertyValueEventMessage::factory()
        ->attributes([
            'id' => $propertyValue->product_property_value_id,
            'product_id' => $propertyValue->product_id,
            'deleted_at' => now()->toJSON(),
        ])
        ->event(ModelEventMessage::UPDATE)
        ->make();

    // Create offer that's linked to property value and must be marked for indexing
    $offerMark = OfferLinkedFactory::create($propertyValue->product_id);
    // Create another offer, to ensure it does not get marked for indexing
    $offerOther = OfferLinkedFactory::create($propertyValue->product_id + 1);

    if ($group) {
        // Prepare offers in one product group
        $groupOffer = OfferLinkedFactory::createGroupedProduct($offerMark->product_id);
        $otherOffer1 = OfferLinkedFactory::create();
        $otherOffer2 = OfferLinkedFactory::createGroupedProduct($otherOffer1->product_id);
    }

    resolve(ListenPublishedPropertyValueAction::class)->execute($message);

    assertDatabaseMissing(ProductPropertyValue::class, ['product_property_value_id' => $propertyValue->product_property_value_id]);
    assertNewModelFieldGreaterThan($offerMark, 'updated_at');
    assertNewModelFieldEquals($offerOther, 'updated_at');

    if ($group) {
        // Validate that offers were marked correctly
        assertNewModelFieldGreaterThan($groupOffer, 'updated_at');
        assertNewModelFieldEquals($otherOffer1, 'updated_at');
        assertNewModelFieldEquals($otherOffer2, 'updated_at');
    }
})->with(FakerProvider::$optionalDataset)->with([true, false]);
